#!/usr/bin/env python
import unittest
import sys
from pathlib import Path
import json
import pandas as pd

import emod_api.campaign as camp
import emodpy_hiv.demographics.HIVDemographics as Demographics
import emodpy_hiv.demographics.DemographicsTemplates as DT

parent = Path(__file__).resolve().parent
sys.path.append(str(parent))
import schema_path_file


class HIVDemographicsTest(unittest.TestCase):
    schema_path = schema_path_file.schema_path

    # region unittest setup and teardown method
    @classmethod
    def setUpClass(cls):
        camp.schema_path = cls.schema_path

    def setUp(self):
        print(f"running {self._testMethodName}:")
        pass

    def tearDown(self):
        print("end of test\n")
        pass
    # endregion

    # region unittests
    def test_from_template_node(self):
        lat = 11
        lon = 22
        pop = 999
        name = "test_name"
        forced_id = 101
        demog = Demographics.from_template_node(lat=lat, lon=lon, pop=pop, name=name, forced_id=forced_id)

        self.assertTrue(isinstance(demog, Demographics.HIVDemographics))
        self.assertEqual(len(demog.implicits), 1)
        self.assertEqual(demog.node_count, 1)

        self.default_test(demog)

        nodes = demog.nodes
        self.assertEqual(len(nodes), 1)
        self.assertEqual(nodes[0].forced_id, forced_id)
        self.assertEqual(nodes[0].name, name)
        self.assertEqual(nodes[0].pop, pop)
        self.assertEqual(nodes[0].lat, lat)
        self.assertEqual(nodes[0].lon, lon)
        pass

    def test_from_pop_csv(self):
        """
                test that verify that the demographics and spatial files can be generated. detailed test is in
        emodapi test_demog_from_pop.py
        """
        pop_filename_in = parent / 'inputs' / 'tiny_facebook_pop_clipped.csv'
        pop_filename_out = parent / "spatial_gridded_pop_dir"
        site = "No_Site"

        demog_filename = parent / 'demographics_from_pop_csv.json'
        spatial_csv_file = pop_filename_out / (site + '_grid.csv')
        spatial_json_file = pop_filename_out / (site + '_grid_id_2_cell_id.json')

        files_to_check = [demog_filename, spatial_csv_file, spatial_json_file]

        for file in files_to_check:
            if file.is_file():  # use missing_ok=True for python 3.8
                file.unlink()

        demog = Demographics.from_pop_csv(pop_filename_in, pop_filename_out=pop_filename_out, site=site)
        self.assertTrue(isinstance(demog, Demographics.HIVDemographics))
        self.assertEqual(len(demog.implicits), 1)
        self.assertEqual(demog.node_count, 5)

        self.default_test(demog)

        demog.generate_file(demog_filename)

        for file in files_to_check:
            self.assertTrue(file.is_file(), msg=f'{file} is not generated.')
        pass

    def test_from_params(self):
        """
            basic test for from_param in emodpy-hiv. more detailed test is in emodapi test_demog.py
        """
        demog_filename = parent / "demographics_from_params.json"
        if demog_filename.is_dir():
            demog_filename.unlink()

        totpop = 9876
        num_nodes = 199
        frac_rural = 0.3
        demog = Demographics.from_params(tot_pop=totpop, num_nodes=num_nodes, frac_rural=frac_rural)
        demog.SetDefaultProperties()
        demog.generate_file(demog_filename)

        self.assertTrue(demog_filename.is_file(), msg=f'{demog_filename} is not generated.')
        with demog_filename.open(mode='r') as demo_file:
            demog_json = json.load(demo_file)
        self.assertEqual(demog_json, demog.raw)

        id_reference = 'from_params'  # hardcoded value
        self.assertEqual(demog.raw['Metadata']['IdReference'], id_reference)
        self.assertEqual(num_nodes, len(demog_json['Nodes']))

        self.default_test(demog)
        pass
    # endregion

    # This is more of a placeholder until implicit config added
    def test_set_society_configs(self):
        config = {"mortality": 1.0, "age_bins": [1.0, 5.0, 10.0]}
        self.assertCountEqual(config, DT.set_society_configs(config))

    def default_test(self, demog):
        defaults = demog.raw['Defaults']
        self.assertIn('NodeAttributes', defaults)
        self.assertIn('IndividualAttributes', defaults)
        self.assertIn('Society', defaults)

    @unittest.skip("Skipping until file ref is fixed in gha")
    def test_mortality(self):
        demog = Demographics.from_template_node( lat=0, lon=0, pop=100000, name=1, forced_id=1 )
        demog.mortality( "inputs/Malawi_male_mortality.csv", "inputs/Malawi_female_mortality.csv", predict_horizon=2060, results_scale_factor=1.0/340.0, csv_out=True )
        male_input = pd.read_csv("inputs/Malawi_male_mortality.csv")
        female_input = pd.read_csv("inputs/Malawi_female_mortality.csv")
        output = demog.raw['Defaults']

        # Check population group consistency
        male_distribution = output['IndividualAttributes']['MortalityDistributionMale']
        female_distribution = output['IndividualAttributes']['MortalityDistributionMale']

        male_pop_groups = male_distribution['NumPopulationGroups'][0]
        self.assertEqual(len(male_distribution['PopulationGroups'][0]), male_pop_groups)

        female_pop_groups = female_distribution['NumPopulationGroups'][0]
        self.assertEqual(len(female_distribution['PopulationGroups'][0]), female_pop_groups)

        # Check result values consistency 
        self.assertEqual(len(male_distribution['ResultValues']), male_distribution['NumPopulationGroups'][0])
        self.assertEqual(len(female_distribution['ResultValues']), female_distribution['NumPopulationGroups'][0])

        # TODO: Check that predictions are aligned with population sizes
        for i in range(19):
            self.assertGreater(male_distribution['ResultValues'][i][0], male_distribution['ResultValues'][i][18]) # Pop sizes ascending, proportion should go down
            self.assertGreater(female_distribution['ResultValues'][i][0], female_distribution['ResultValues'][i][18])

        # Check prediction horizon is honored
        self.assertLessEqual(max(male_distribution['PopulationGroups'][1]), 2060)
        self.assertLessEqual(max(female_distribution['PopulationGroups'][1]), 2060)

        # Check results scale factor is consistent with parameters
        self.assertEqual(male_distribution['ResultScaleFactor'], 1.0/340.0)
        self.assertEqual(female_distribution['ResultScaleFactor'], 1.0/340.0)

        # TODO: check the mortality distribution against csv data 

if __name__ == '__main__':
    unittest.main()

