#!/usr/bin/env python
import unittest
import sys
from pathlib import Path
import json
import os

import emod_api.campaign as camp
import emodpy_hiv.interventions.outbreak as ob
import emodpy_hiv.interventions.art as art
import emodpy_hiv.interventions.artdropout as artdropout
import emodpy_hiv.interventions.artstagingbycd4agnosticdiag as artstageagnosticdiag
import emodpy_hiv.interventions.artstagingbycd4diag as artstage4diag
import emodpy_hiv.interventions.drawblood as drawblood
import emodpy_hiv.interventions.malecirc as malecirc
import emodpy_hiv.interventions.modcoinf as modcoinf  
import emodpy_hiv.interventions.pmtct as pmtct
import emodpy_hiv.interventions.prep as prep
import emodpy_hiv.interventions.random as random
import emodpy_hiv.interventions.rapiddiag as rapiddiag
import emodpy_hiv.interventions.sigmoiddiag as sigmoiddiag
import emodpy_hiv.interventions.stipostdebut as stipostdebut
import emodpy_hiv.interventions.yearandsexdiag as yearandsexdiag




parent = Path(__file__).resolve().parent
sys.path.append(str(parent))
import schema_path_file


class HIVInterventionTest(unittest.TestCase):
    schema_path = schema_path_file.schema_path
    is_debugging = True

    # region unittest setup and teardown method
    @classmethod
    def setUpClass(cls):
        camp.schema_path = cls.schema_path

    def setUp(self):
        print(f"running {self._testMethodName}:")
        self.output_filename = ""
        pass

    def tearDown(self):
        if os.path.exists(self.output_filename) and not self.is_debugging:
            os.remove(self.output_filename)

        print("end of test\n")
        pass
    # endregion

    # region outbreak
    def test_outbreak_new_intervention(self):
        timestep = 365
        coverage = 0.05
        event = ob.new_intervention(timestep=timestep, camp=camp, coverage=coverage)
        self.assertEqual(event.Start_Day, timestep)
        self.assertEqual(event.Event_Coordinator_Config.Intervention_Config['class'], 'OutbreakIndividual')
        self.assertEqual(event.Event_Coordinator_Config.Demographic_Coverage, coverage)
    # endregion

    # region ART
    def test_art_new_default(self):
        start_day = 1
        coverage = 1.0
        node_ids = []
        event = art.new_intervention_event(camp=camp, start_day=start_day, coverage=coverage, node_ids=None)

        self.assertEqual(event.Start_Day, start_day)
        self.assertEqual(event.Event_Coordinator_Config.Intervention_Config['class'], 'AntiretroviralTherapy')
        self.assertEqual(event.Event_Coordinator_Config.Demographic_Coverage, coverage)

    def test_art_new_custom(self):
        start_day = 5
        coverage = 0.5
        event = art.new_intervention_event(camp=camp, start_day=start_day, coverage=coverage, node_ids=None)

        self.assertEqual(event.Start_Day, start_day)
        self.assertEqual(event.Event_Coordinator_Config.Intervention_Config['class'], 'AntiretroviralTherapy')
        self.assertEqual(event.Event_Coordinator_Config.Demographic_Coverage, coverage)

    def test_art_as_file(self):
        start_day = 5
        coverage = 1.0 # No way to configure
        event = art.new_intervention_as_file(camp=camp, start_day=start_day, filename=None)

        self.output_filename = "AntiRetroviralTherapy.json"
        with open(self.output_filename) as art_file:
            event = json.load(art_file)

        event = event['Events'][0]

        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'AntiretroviralTherapy')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)

    def test_art_dropout_custom(self):
        start_day = 2
        coverage = 0.8
        node_ids = [1, 2] # Gonna add this check later

        event = artdropout.new_intervention_event(camp, start_day, coverage, node_ids)
        
        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'ARTDropout')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)

    def test_art_dropout_as_file(self):
        start_day = 2
        coverage = 1.0
        node_ids = [1, 2] # Gonna add this check later

        self.output_filename = "ARTDropout.json"

        event = artdropout.new_intervention_as_file(camp, start_day, self.output_filename)
        
        with open(self.output_filename) as art_dropout_file:
            event = json.load(art_dropout_file)

        event = event['Events'][0]

        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'ARTDropout')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)

    # end region

    def test_artstaging_agnostic_custom(self):
        start_day = 2
        coverage = 0.8
        pos_event = "BleepBloop"
        neg_event = "BloopBleep"
        abw_tvmap = {"Stage 1": 0.4, "Stage 2": 0.5, "Stage 3": 0.1}
        abt_tvmap = {"TB 1": 0.3, "TB 2": 0.2, "TB 3": 0.5}
        abp_tvmap = {"Not pregnant": 0.95, "Pregnant": 0.05}
        cua_tvmap = {"Under 5": 0.3, "Over 5": 0.7}
        cbt_tvmap = {"Has TB": 0.8, "No TB": 0.2}
        cbw_tvmap = {"Child Stage 1": 0.1, "Child Stage 2": 0.2, "Child Stage 3": 0.7}


        event = artstageagnosticdiag.new_intervention_event(camp, pos_event, neg_event, abp_tvmap, abt_tvmap, abw_tvmap, cua_tvmap, cbt_tvmap, cbw_tvmap,start_day, coverage)
        intervention_config = event['Event_Coordinator_Config']['Intervention_Config']
        
        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(intervention_config['class'], 'HIVARTStagingCD4AgnosticDiagnostic')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(intervention_config['Negative_Diagnosis_Event'], "GP_EVENT_001")
        self.assertEqual(intervention_config['Positive_Diagnosis_Event'], "GP_EVENT_000")

        abp_map_values = intervention_config['Adult_By_Pregnant']
        self.assertEqual(abp_map_values['Times'], ['Not pregnant', 'Pregnant'])
        self.assertEqual(abp_map_values['Values'], [0.95, 0.05])

        abw_map_values = intervention_config['Adult_By_TB']
        self.assertEqual(abw_map_values['Times'], ['TB 1', 'TB 2', "TB 3"])
        self.assertEqual(abw_map_values['Values'], [0.3, 0.2, 0.5])

        abt_map_values = intervention_config['Adult_By_TB']
        self.assertEqual(abt_map_values['Times'], ['TB 1', 'TB 2', "TB 3"])
        self.assertEqual(abt_map_values['Values'], [0.3, 0.2, 0.5])

        cua_map_values = intervention_config['Child_Treat_Under_Age_In_Years_Threshold']
        self.assertEqual(cua_map_values['Times'], ['Under 5', 'Over 5'])
        self.assertEqual(cua_map_values['Values'], [0.3, 0.7])

        cbt_map_values = intervention_config['Child_By_TB']
        self.assertEqual(cbt_map_values['Times'], ['Has TB', 'No TB'])
        self.assertEqual(cbt_map_values['Values'], [0.8, 0.2])

    def test_artstaging_agnostic_file(self):
        start_day = 2
        coverage = 1.0
        node_ids = [1, 2] # Gonna add this check later

        self.output_filename = "Art_Staging_Agnostic.json"
        event = artstageagnosticdiag.new_intervention_as_file(camp, start_day, self.output_filename)
        
        with open(self.output_filename) as art_staging_agnostic_file:
            event = json.load(art_staging_agnostic_file)

        event = event['Events'][0]


        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'HIVARTStagingCD4AgnosticDiagnostic')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Negative_Diagnosis_Event'], "HIVTestedNegative")
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Positive_Diagnosis_Event'], "HIVTestedPositive")

    def test_artstagingbycd4diag_custom(self):
        start_day = 2
        coverage = 0.8
        pos_event = "BleepBloop"
        neg_event = "BloopBleep"
        abw_tvmap = {"Stage 1": 0.4, "Stage 2": 0.5, "Stage 3": 0.1}
        abt_tvmap = {"TB 1": 0.3, "TB 2": 0.2, "TB 3": 0.5}
        abp_tvmap = {"Not pregnant": 0.95, "Pregnant": 0.05}
        cua_tvmap = {"Under 5": 0.3, "Over 5": 0.7}
        cbt_tvmap = {"Has TB": 0.8, "No TB": 0.2}
        cbw_tvmap = {"Child Stage 1": 0.1, "Child Stage 2": 0.2, "Child Stage 3": 0.7}

        event = artstageagnosticdiag.new_intervention_event(camp, pos_event, neg_event, abp_tvmap, abt_tvmap, abw_tvmap, cua_tvmap, cbt_tvmap, cbw_tvmap, start_day, coverage)
        intervention_config = event['Event_Coordinator_Config']['Intervention_Config']

        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(intervention_config['class'], 'HIVARTStagingCD4AgnosticDiagnostic')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(intervention_config['Negative_Diagnosis_Event'], "GP_EVENT_001")
        self.assertEqual(intervention_config['Positive_Diagnosis_Event'], "GP_EVENT_000")

        abp_map_values = intervention_config['Adult_By_Pregnant']
        self.assertEqual(abp_map_values['Times'], ['Not pregnant', 'Pregnant'])
        self.assertEqual(abp_map_values['Values'], [0.95, 0.05])

        abw_map_values = intervention_config['Adult_By_TB']
        self.assertEqual(abw_map_values['Times'], ['TB 1', 'TB 2', "TB 3"])
        self.assertEqual(abw_map_values['Values'], [0.3, 0.2, 0.5])

        abt_map_values = intervention_config['Adult_By_TB']
        self.assertEqual(abt_map_values['Times'], ['TB 1', 'TB 2', "TB 3"])
        self.assertEqual(abt_map_values['Values'], [0.3, 0.2, 0.5])

        cua_map_values = intervention_config['Child_Treat_Under_Age_In_Years_Threshold']
        self.assertEqual(cua_map_values['Times'], ['Under 5', 'Over 5'])
        self.assertEqual(cua_map_values['Values'], [0.3, 0.7])

        cbt_map_values = intervention_config['Child_By_TB']
        self.assertEqual(cbt_map_values['Times'], ['Has TB', 'No TB'])
        self.assertEqual(cbt_map_values['Values'], [0.8, 0.2])

    def test_artstagingbycd4diag_as_file(self):
        start_day = 2
        coverage = 1.0
        node_ids = [1, 2] # Gonna add this check later

        self.output_filename = "artstagingbycd4diag.json"

        event = artstageagnosticdiag.new_intervention_as_file(camp, start_day, self.output_filename)
        
        with open(self.output_filename) as art_staging_cd4diag_file:
            event = json.load(art_staging_cd4diag_file)

        event = event['Events'][0]


        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'HIVARTStagingCD4AgnosticDiagnostic')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Negative_Diagnosis_Event'], "HIVTestedNegative")
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Positive_Diagnosis_Event'], "HIVTestedPositive")

    def test_draw_blood_custom(self):
        pos_event = "BleepBloop"
        neg_event = "BloopBleep"
        start_day = 2
        coverage = 0.8

        event = drawblood.new_intervention_event(camp, pos_event, neg_event, start_day, coverage)

        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'HIVDrawBlood')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Negative_Diagnosis_Event'], "GP_EVENT_001")
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Positive_Diagnosis_Event'], "GP_EVENT_000")

    def test_draw_blood_file(self):
        start_day = 2
        coverage = 1.0

        self.output_filename = "blooddraw.json"

        event = artstageagnosticdiag.new_intervention_as_file(camp, start_day, self.output_filename)
        
        with open(self.output_filename) as blood_file:
            event = json.load(blood_file)

        event = event['Events'][0]


        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'HIVARTStagingCD4AgnosticDiagnostic')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Negative_Diagnosis_Event'], "HIVTestedNegative")
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Positive_Diagnosis_Event'], "HIVTestedPositive")

    def test_malecirc_custom(self):
        start_day = 2
        coverage = 0.8
        node_ids = [1, 2] # Gonna add this check later

        event = malecirc.new_intervention_event(camp, start_day, coverage, node_ids)
        
        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'MaleCircumcision')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)

    def test_malecirc_file(self):
        start_day = 2
        coverage = 1.0
        node_ids = [1, 2] # Gonna add this check later

        self.output_filename = "malecirc.json"

        event = malecirc.new_intervention_as_file(camp, start_day, self.output_filename)
        
        with open(self.output_filename) as circumcision_file:
            event = json.load(circumcision_file)

        event = event['Events'][0]
        
        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'MaleCircumcision')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)

    def test_modcoinf_custom(self):
        start_day = 2
        coverage = 0.8
        node_ids = [1, 2] # Gonna add this check later

        event = modcoinf.new_intervention_event(camp, start_day, coverage, node_ids)
        
        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'ModifyStiCoInfectionStatus')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
    
    def test_modcoinf_file(self):
        start_day = 2
        coverage = 1.0
        node_ids = [1, 2] # Gonna add this check later

        self.output_filename = "modcoinf.json"

        event = modcoinf.new_intervention_as_file(camp, start_day, self.output_filename)
        
        with open(self.output_filename) as modcoinf_file:
            event = json.load(modcoinf_file)

        event = event['Events'][0]
        
        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'ModifyStiCoInfectionStatus')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)

    def test_pmtct_custom(self):
        start_day = 2
        coverage = 0.8
        node_ids = [1, 2] # Gonna add this check later

        event = pmtct.new_intervention_event(camp, start_day, coverage, node_ids)
        
        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'PMTCT')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)

    def test_pmtct_file(self):
        start_day = 2
        coverage = 1.0
        node_ids = [1, 2] # Gonna add this check later

        self.output_filename = "pmtct.json"

        event = pmtct.new_intervention_as_file(camp, start_day, self.output_filename)
        
        with open(self.output_filename) as pmtct_file:
            event = json.load(pmtct_file)

        event = event['Events'][0]
        
        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'PMTCT')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)

    def test_prep_custom(self):
        start_day = 2
        coverage = 0.8
        node_ids = [1, 2] # Gonna add this check later

        event = prep.new_intervention_event(camp, start_day, coverage, node_ids)
        
        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'ControlledVaccine')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)

    def test_prep_file(self):
        start_day = 2
        coverage = 1.0
        node_ids = [1, 2] # Gonna add this check later

        self.output_filename = "prep.json"
        event = prep.new_intervention_as_file(camp, start_day, self.output_filename)
        
        with open(self.output_filename) as prep_file:
            event = json.load(prep_file)

        event = event['Events'][0]
        
        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'ControlledVaccine')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)

    def test_random_custom(self):
        start_day = 2
        coverage = 0.8
        choices = {"Yes":0.4, "No":0.6}

        event = random.new_intervention_event(camp, choices, start_day, coverage)

        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'HIVRandomChoice')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Choices'], choices)

    def test_random_file(self):
        start_day = 2
        coverage = 1.0
        choices = {"Yes":0.4, "No":0.6}

        self.output_filename = "random.json"
        event = random.new_intervention_as_file(camp, start_day, choices, self.output_filename)
        
        with open(self.output_filename) as random_file:
            event = json.load(random_file)

        event = event['Events'][0]


        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'HIVRandomChoice')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Choices'], choices)

    def test_rapiddiag_custom(self):
        pos_event = "BleepBloop"
        neg_event = "BloopBleep"
        start_day = 2
        coverage = 0.8

        event = rapiddiag.new_intervention_event(camp, pos_event, neg_event, start_day, coverage)

        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'HIVRapidHIVDiagnostic')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Negative_Diagnosis_Event'], "GP_EVENT_001")
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Positive_Diagnosis_Event'], "GP_EVENT_000")

    def test_rapiddiag_file(self):
        start_day = 2
        coverage = 1.0

        self.output_filename = "rapiddiag.json"

        event = rapiddiag.new_intervention_as_file(camp, start_day, self.output_filename)
        
        with open(self.output_filename) as rapiddiag_file:
            event = json.load(rapiddiag_file)

        event = event['Events'][0]

        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'HIVRapidHIVDiagnostic')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Negative_Diagnosis_Event'], "HIVTestedNegative")
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Positive_Diagnosis_Event'], "HIVTestedPositive")

    def test_sigmoiddiag_custom(self):
        pos_event = "BleepBloop"
        neg_event = "BloopBleep"
        start_day = 2
        coverage = 0.8
        ramp_min = 0.2
        ramp_max = 0.8
        ramp_midyear = 2005
        ramp_rate = 0.8

        event = sigmoiddiag.new_intervention_event(camp, pos_event, neg_event, ramp_min, ramp_max, ramp_midyear, ramp_rate, start_day, coverage)

        intervention_config = event['Event_Coordinator_Config']['Intervention_Config']

        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(intervention_config['class'], 'HIVSigmoidByYearAndSexDiagnostic')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(intervention_config['Negative_Diagnosis_Event'], "GP_EVENT_001")
        self.assertEqual(intervention_config['Positive_Diagnosis_Event'], "GP_EVENT_000")
        self.assertEqual(intervention_config['Ramp_Max'], ramp_max)
        self.assertEqual(intervention_config['Ramp_Min'], ramp_min)
        self.assertEqual(intervention_config['Ramp_MidYear'], ramp_midyear)
        self.assertEqual(intervention_config['Ramp_Rate'], ramp_rate)

    def test_sigmoiddiag_file(self):
        start_day = 2
        coverage = 1.0

        self.output_filename = "sigmoiddiag.json"

        event = sigmoiddiag.new_intervention_as_file(camp, start_day, self.output_filename)
        
        with open(self.output_filename) as sigmoiddiag_file:
            event = json.load(sigmoiddiag_file)

        event = event['Events'][0]

        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'HIVSigmoidByYearAndSexDiagnostic')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Negative_Diagnosis_Event'], "HIVTestedNegative")
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Positive_Diagnosis_Event'], "HIVTestedPositive")

    def test_stipostdebut_custom(self):
        pos_event = "BleepBloop"
        neg_event = "BloopBleep"
        start_day = 2
        coverage = 0.8

        event = stipostdebut.new_intervention_event(camp, pos_event, neg_event, start_day, coverage)

        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'STIIsPostDebut')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Negative_Diagnosis_Event'], "GP_EVENT_001")
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Positive_Diagnosis_Event'], "GP_EVENT_000")

    def test_stipostdebut_file(self):
        start_day = 2
        coverage = 1.0

        self.output_filename = "stipostdebut.json"

        event = stipostdebut.new_intervention_as_file(camp, start_day, self.output_filename)
        
        with open(self.output_filename) as stipostdebut_file:
            event = json.load(stipostdebut_file)

        event = event['Events'][0]

        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['class'], 'STIIsPostDebut')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Negative_Diagnosis_Event'], "HIVTestedNegative")
        self.assertEqual(event['Event_Coordinator_Config']['Intervention_Config']['Positive_Diagnosis_Event'], "HIVTestedPositive")

    def test_yearandsexdiag_custom(self):
        pos_event = "BleepBloop"
        neg_event = "BloopBleep"
        start_day = 2
        coverage = 0.8
        tvmap = {"Bing":0, "Bong":1}

        event = yearandsexdiag.new_intervention_event(camp, pos_event, neg_event, tvmap, start_day, coverage)
        intervention_config = event['Event_Coordinator_Config']['Intervention_Config']

        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(intervention_config['class'], 'HIVPiecewiseByYearAndSexDiagnostic')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(intervention_config['Negative_Diagnosis_Event'], "GP_EVENT_001")
        self.assertEqual(intervention_config['Positive_Diagnosis_Event'], "GP_EVENT_000")
        timemap = intervention_config['Time_Value_Map']
        self.assertEqual(timemap['Times'], ['Bing', 'Bong'])
        self.assertEqual(timemap['Values'], [0, 1])

    def test_yearandsexdiag_file(self):
        start_day = 2
        coverage = 1.0
        tvmap = {"Bing":0, "Bong":1}

        self.output_filename = "yearandsexdiag.json"

        event = yearandsexdiag.new_intervention_as_file(camp, start_day, tvmap, self.output_filename)
        
        with open(self.output_filename) as yearandsexdiag_file:
            event = json.load(yearandsexdiag_file)

        event = event['Events'][0]
        intervention_config = event['Event_Coordinator_Config']['Intervention_Config']

        self.assertEqual(event['Start_Day'], start_day)
        self.assertEqual(intervention_config['class'], 'HIVPiecewiseByYearAndSexDiagnostic')
        self.assertEqual(event['Event_Coordinator_Config']['Demographic_Coverage'], coverage)
        self.assertEqual(intervention_config['Negative_Diagnosis_Event'], "HIVTestedNegative")
        self.assertEqual(intervention_config['Positive_Diagnosis_Event'], "HIVTestedPositive")

        timemap = intervention_config['Time_Value_Map']
        self.assertEqual(timemap['Times'], ['Bing', 'Bong'])
        self.assertEqual(timemap['Values'], [0, 1])

if __name__ == '__main__':
    unittest.main()